from time import time
from math import log
from math import ceil
from pickle import load
from os.path import exists
from pickle import dump

from funki.data import max_width

import funki.text_colours as tc

φ_a = lambda x: ('\n' if '\n' in x else '') + f'{x}'

_fail = f"{'FAIL':^8}"
_pass = f'{"PASS":^8}'

def φ(x):
  max_length = 24*450

  result = φ_a(x)
  if len(result) > max_length:
    result = result[:max_length] + '...'
  
  return result

def _duration_bar(duration_ms):
  # width = min(round(duration_ms / 0.03125), max_width)
  width = min(max(0, ceil(log(1000*duration_ms, 2))), max_width)
  return (width*'█' + (max_width-width)*' ', width)

def _duration(t_0, name):
  T_ms = (time()-t_0)*1000
  
  durations = {}

  if exists('durations.pickle'):
    with open('durations.pickle', 'rb') as φ:
      durations = load(φ)
  
  if name in durations:
    durations[name] = (durations[name] *(max_width-1) + T_ms)/(max_width)
  else:
    durations[name] = T_ms

  with open('durations.pickle', 'wb') as φ:
    dump(durations, φ)

  dbar, w = _duration_bar(T_ms)
  
  style = tc.bright_red
  if w < 16: style = tc.bright_magenta
  if w < 12: style = tc.bright_blue
  if w < 8: style = tc.bright_cyan
  if w < 4: style = tc.bright_green

  return style(f'{dbar} {T_ms:8.3f}')

def fail(name, xyz, mode, time_started, max_filename_length):
  print('\n'.join([
    f"\r|{tc.danger(_fail)}| {name:{max_filename_length}} | {_duration(time_started, name)} |",
    f"{tc.danger('mode:')} {mode}",
    f"{tc.primary('Input       x_args:   ')} {φ(str(xyz['x_args']))}",
    f"{tc.primary('Expectation y_return: ')} {φ(str(xyz['y_return']))}",
    f"{tc.primary('Observation z_return: ')} {φ(str(xyz['z_return']))}",
    f"{tc.secondary('Input       x_stdi: ')} {φ(str(xyz['x_stdi']))}",
    f"{tc.secondary('Expectation y_stdo: ')} {φ(str(xyz['y_stdo']))}",
    f"{tc.secondary('Observation z_stdo: ')} {φ(str(xyz['z_stdo']))}",
  ]))
  return False

def property_failure(name, mode, time_started, max_filename_length):
  print('\n'.join([
    f"\r|{tc.danger(_fail)}| {name:{max_filename_length}} | {_duration(time_started, name)} |",
    f"{tc.danger('mode:')} {mode}",
  ]))
  return False

def success(name, time_started, max_filename_length):
  print(
    f"\r|{tc.success(_pass)}| {name:{max_filename_length}} | {_duration(time_started, name)} |",
  )
  return True
